/****************************************************************************************/
/*                                                                                      */
/*  Project     :  1D-BRNN                                                              */
/*  Release     :  3.3                                                                  */
/*                                                                                      */
/*  File        :  Dataset.h                                                            */
/*  Description :  Dataset of sequences in input of the program                         */
/*                                                                                      */
/*  Author(s)   :  Christophe Magnan  (2013) - New generic version                      */
/*              :  Jianlin Cheng      (2003) - New custom version for SCRATCH           */
/*              :  Gianluca Pollastri (2001) - Customized version for SCRATCH           */
/*              :  Paolo Frasconi     (1997) - Initial generic version                  */
/*                                                                                      */
/*  Copyright   :  Institute for Genomics and Bioinformatics                            */
/*                 University of California, Irvine                                     */
/*                                                                                      */
/*  Modified    :  2015/07/01                                                           */
/*                                                                                      */
/****************************************************************************************/

#include "Sequence.h"

class Dataset
{

  public:

  // Dataset content

  int Num_Sequences;        // Number of sequences in the dataset
  int Num_Positions;        // Number of positions in the sequences
  int Num_Features;         // Number of features for each position
  int Num_Classes;          // Number of possible target classes
  Sequence** Sequences;     // Sequences in the dataset

  // Interface

  Dataset();
  ~Dataset();
  void load(char* filein);
  void alloc_propagation(int out_fwd,int out_bwd);
  void alloc_backpropagation(int out_fwd,int out_bwd);
  void write_predictions(char* fileout);
  void display_confusion();
  float empirical_risk();
  void check_viability();

};

