/****************************************************************************************/
/*                                                                                      */
/*  Project     :  1D-BRNN                                                              */
/*  Release     :  3.3                                                                  */
/*                                                                                      */
/*  File        :  Predict_Multi_Models.cpp                                             */
/*  Description :  Predictions of several BRNN models on a dataset of sequences         */
/*                                                                                      */
/*  Author(s)   :  Christophe Magnan  (2013) - New generic version                      */
/*              :  Jianlin Cheng      (2003) - New custom version for SCRATCH           */
/*              :  Gianluca Pollastri (2001) - Customized version for SCRATCH           */
/*              :  Paolo Frasconi     (1997) - Initial generic version                  */
/*                                                                                      */
/*  Copyright   :  Institute for Genomics and Bioinformatics                            */
/*                 University of California, Irvine                                     */
/*                                                                                      */
/*  Modified    :  2015/07/01                                                           */
/*                                                                                      */
/****************************************************************************************/

#include "Model.h"

void predict_multi(int Num_Models,Model** BRNNs,Dataset* DATA,char* fileout)
{
  int i,j,c,p,l,n=DATA->Num_Classes; Sequence* s;
  for(i=0;i<DATA->Num_Sequences;i++)
  { s=DATA->Sequences[i]; l=s->Num_Positions; float** sum;
    sum=new float*[l]; for(p=0;p<l;p++){ sum[p]=new float[n];
    for(c=0;c<n;c++){ sum[p][c]=0.0; } }
    for(j=0;j<Num_Models;j++){ BRNNs[j]->propagate(s);
    for(p=0;p<l;p++){ for(c=0;c<n;c++){
    sum[p][c]+=s->Outputs_MAIN[p][c]; } } }
    for(p=0;p<l;p++){ float max=0; int cl=0;
    for(c=0;c<n;c++){ float pb=sum[p][c];
    s->Outputs_MAIN[p][c]=pb/(float)Num_Models;
    if(pb>max){ max=pb; cl=c; } }
    s->Predictions[p]=cl; } }
  DATA->write_predictions(fileout);
}

int main(int argc,char** argv)
{
  // Script Arguments
  char usage[40]=" dataset models_list predictions\n\n";
  if(argc!=4){ cerr << "\n  Script usage : " << argv[0] << usage; exit(1); }

  // Input Dataset
  Dataset* data=new Dataset(); data->load(argv[1]);

  // Models List
  Model** models; int num_models=0; char model_file[1024];
  filebuf in_buffer; int max_out_fwd=0; int max_out_bwd=0;
  if(in_buffer.open(argv[2],ios::in)!=0)
  { istream is(&in_buffer); is >> num_models;
    if(num_models<1){ cerr << "Incorrect format, exit."; exit(1); }

    // Input Models
    models=new Model*[num_models]; for(int i=0;i<num_models;i++){
    is >> model_file; models[i]=new Model(); models[i]->load(model_file);
    int of=models[i]->Num_Outputs_FWD; if(of>max_out_fwd){ max_out_fwd=of; }
    int ob=models[i]->Num_Outputs_BWD; if(ob>max_out_bwd){ max_out_bwd=ob; }

    // Data Compatibility
    int c=1; if(data->Num_Features!=models[i]->Num_Features){ c=0; }
    if(data->Num_Classes!=models[i]->Num_Classes){ c=0; }
    if(!c){ cerr << "Incompatible data, cannot proceed\n"; exit(1); } }

  // Close Models List
  in_buffer.close(); }
  else{ cerr << "Cannot read file \"" << argv[2] << "\"\n"; exit(1); }

  // Memory allocation
  data->alloc_propagation(max_out_fwd,max_out_bwd);

  // Predictions
  predict_multi(num_models,models,data,argv[3]); return 1;
}

