/****************************************************************************************/
/*                                                                                      */
/*  Project     :  1D-BRNN                                                              */
/*  Release     :  3.3                                                                  */
/*                                                                                      */
/*  File        :  Predict_Single_Model.cpp                                             */
/*  Description :  Predictions of a single BRNN model on a dataset of sequences         */
/*                                                                                      */
/*  Author(s)   :  Christophe Magnan  (2013) - New generic version                      */
/*              :  Jianlin Cheng      (2003) - New custom version for SCRATCH           */
/*              :  Gianluca Pollastri (2001) - Customized version for SCRATCH           */
/*              :  Paolo Frasconi     (1997) - Initial generic version                  */
/*                                                                                      */
/*  Copyright   :  Institute for Genomics and Bioinformatics                            */
/*                 University of California, Irvine                                     */
/*                                                                                      */
/*  Modified    :  2015/07/01                                                           */
/*                                                                                      */
/****************************************************************************************/

#include "Model.h"

void predict_single(Model* BRNN,Dataset* DATA,char* fileout)
{
  for(int i=0;i<DATA->Num_Sequences;i++){
  BRNN->propagate(DATA->Sequences[i]); }
  DATA->write_predictions(fileout);
}

int main(int argc,char** argv)
{
  // Script Arguments
  char usage[40]=" dataset model predictions\n\n";
  if(argc!=4){ cerr << "\n  Script usage : " << argv[0] << usage; exit(1); }

  // Input Dataset
  Dataset* data=new Dataset(); data->load(argv[1]);

  // Input Model
  Model* brnn=new Model(); brnn->load(argv[2]);

  // Data Compatibility
  int c=1; if(data->Num_Features!=brnn->Num_Features){ c=0; }
  if(data->Num_Classes!=brnn->Num_Classes){ c=0; }
  if(!c){ cerr << "Incompatible data, cannot proceed\n"; exit(1); }

  // Memory allocation
  data->alloc_propagation(brnn->Num_Outputs_FWD,brnn->Num_Outputs_BWD);

  // Predictions
  predict_single(brnn,data,argv[3]); return 1;
}

